<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skrooge_scheduled" version="11">
<MenuBar>
  <Menu name="edit"><text>Edit</text>
	  <Action name="schedule_operation" group="edit_group_1"/>
	  <Action name="skip_scheduled_operations" group="edit_group_1"/>
	  <Action name="edit_assign_schedule" group="edit_group_1"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>
</kpartgui>
